@@ -3,21 +3,18 @@
 #
 #  (C) Paul Evans, 2008-2013 -- leonerd@leonerd.org.uk
 
-package # hide from CPAN
-   IO::Async::Internals::Connector;
+package IO::Async::Internals::Connector;
 
 use strict;
 use warnings;
 
-our $VERSION = '0.60_004';
-
-use Scalar::Util qw( weaken );
+our $VERSION = '0.59';
 
 use POSIX qw( EINPROGRESS );
 use Socket qw( SOL_SOCKET SO_ERROR );
 
 use Future;
-use Future::Utils 0.18 qw( try_repeat_until_success );
+use Future::Utils qw( repeat_until_success );
 
 use IO::Async::OS;
 
@@ -33,8 +30,9 @@ sub new
 
    my $loop = delete $params{loop} or croak "Expected a 'loop'";
 
-   my $self = bless {}, $class;
-   weaken( $self->{loop} = $loop );
+   my $self = bless {
+      loop => $loop,
+   }, $class;
 
    return $self;
 }
@@ -85,7 +83,7 @@ sub _connect_addresses
 
    my ( $connecterr, $binderr, $socketerr );
 
-   my $future = try_repeat_until_success {
+   my $future = repeat_until_success {
       my $addr = shift;
       my ( $family, $socktype, $protocol, $localaddr, $peeraddr ) =
          @{$addr}{qw( family socktype protocol localaddr peeraddr )};
@@ -139,7 +137,7 @@ sub _connect_addresses
          sub { $loop->unwatch_io( handle => $sock, on_write_ready => 1 ); }
       );
       return $f;
-   } foreach => $addrlist;
+   } foreach => $addrlist, return => $loop->new_future;
 
    return $future->or_else( sub {
       return $future->new->fail( "connect: $connecterr", connect => connect => $connecterr )
@@ -209,6 +207,11 @@ sub connect
    }
 
    return Future->needs_all( $peeraddrfuture, $localaddrfuture )
+      ->or_else( sub {
+         my $f = shift;
+         my ( $failure, @args ) = $f->failure;
+         return $loop->new_future->fail( $failure, resolve => @args );
+      } )
       ->and_then( sub {
          my @peeraddrs  = $peeraddrfuture->get;
          my @localaddrs = $localaddrfuture->get;
